/*****************************************************************************
 * BLx_Test.c
 *
 * This file demonstrates how to boot a secure loader stream via SPI on an
 * open part.
 *
 * The BMODE pins must be set to SPI Master boot.
 *
 * This application code illustrates how to use the adi_rom_Boot ROM API
 * function to configure an open part for secure boot. This allows testing of
 * private keys, and a signed boot stream.
 *
 * The application works by supplying a hook function to the ROM API that
 * configures the kernel to boot in secure mode, and sets the appropriate
 * keys that would otherwise be read from OTP
 *
 * The example consists of three main components:
 *      setupMemMappedSPI() used by the hook function to workaround kernel anomalies
 *      ConfigureForSecureBoot() configures the kenrel for secure boot ( assigns keys )
 *      main() the calling application to be loaded into L1 via the emulator
 *             this function will initiate the secure boot
 *
 *
 *
 * Copyright (c) 2014 Analog Devices, Inc. All Rights Reserved.
 *****************************************************************************/

#include "adi_initialize.h"
#include <sys/platform.h>

#if   defined(__ADSPBF70x__) && __ADSPBF70x__==1
#include <bfrom.h>
#elif defined(__ADSPSC5xx__) && __ADSPSC5xx__==1
#include <cdefSC58x_rom.h>
#endif



/**
 * @brief   setup kernel for memory mapped SPI mode
 * @param   pBootConfig boot configuration structure
 */
void setupMemoryMappedSPI(ADI_ROM_BOOT_CONFIG * pBootConfig) {

  /* kernel anomaly workarounds for SPI */
  /* memory mapped mode must be used when using the API for secure boot in an open part, this is not required for a normal boot */

  ADI_SPI_TypeDef *pSpiRegs = ( ADI_SPI_TypeDef *)pBootConfig->pPeripheralBase;
  ADI_ROM_BOOT_SPI *pSpiStruct  = ( ADI_ROM_BOOT_SPI *)pBootConfig->pModeData;
  pBootConfig->pDmaBaseRegister = pADI_MDMA0_DST;

  pBootConfig->dControlValue = BITM_SPI_CTL_EN | BITM_SPI_CTL_MSTR | (0 << BITP_SPI_CTL_SIZE) | BITM_SPI_CTL_MMSE | BITM_SPI_CTL_ASSEL;
  pSpiRegs->TXCTL |= BITM_SPI_TXCTL_TEN | BITM_SPI_TXCTL_TTI;
  pSpiRegs->RXCTL |= BITM_SPI_RXCTL_REN | BITM_SPI_RXCTL_RTI;
  pSpiRegs->CTL = pBootConfig->dControlValue | pSpiStruct->nRxCtl;
  pBootConfig->pSource = (void *)((uint8_t)pBootConfig->pSource + pSpiStruct->pXIPAddress);

  /* change dBootCommand to indicate now using the SPI XIP mode, this is needed for the load function */
  pBootConfig->dBootCommand &= (!BITM_ROM_BCMD_DEVICE);
  pBootConfig->dBootCommand |= ENUM_ROM_BCMD_DEVICE_SPIXIP;

}


/*****************************************************
 *  Configure functions for the various image types
 *****************************************************/
#include "secure_boot_public_key.h"
#include "wrapper_key.h"
#include "encrypt_key.h"
void configureBlp(ADI_ROM_BOOT_CONFIG* pBootConfig) {
    uint8_t * pKey = (uint8_t *)&pBootConfig->publicKey;
    uint8_t i;

    for(i=0; i<56; i++) {
      pKey[i] = secure_boot_public_key[i];
    }

}
void configureBlx(ADI_ROM_BOOT_CONFIG* pBootConfig) {

    uint8_t * pKey = (uint8_t *)&pBootConfig->publicKey;
    uint8_t * aes  = (uint8_t *)&pBootConfig->aesKey;
    uint8_t i;

    for(i=0; i<56; i++) {
      pKey[i] = secure_boot_public_key[i];
    }

    for(i=0; i<16; i++) {
      aes[i] = encrypt_key[i];
    }
}
void configureBlw(ADI_ROM_BOOT_CONFIG* pBootConfig) {

    uint8_t * pKey     = (uint8_t *)&pBootConfig->publicKey;
    uint8_t * aesWrap  = (uint8_t *)&pBootConfig->aesWrapKey;
    uint8_t i;

    for(i=0; i<56; i++) {
      pKey[i] = secure_boot_public_key[i];
    }

    for(i=0; i<16; i++) {
      aesWrap[i] = wrapper_key[i];
    }
}

/**
 * ConfigureForSecureBoot
 * @param pBootConfig boot configuration structure
 * @param cause the reason the hook was called
 * @brief The hook function is called twice by the ROM after configuration and after initialization
 *        Here we will setup the boot Kernel to do a secure boot.
 *
 * */
int32_t ConfigureForSecureBoot(ADI_ROM_BOOT_CONFIG * pBootConfig, ROM_HOOK_CALL_CAUSE cause) {


  if (cause == ROM_HOOK_CALL_INIT_COMPLETE) {
    /* executed after kernel has completed boot peripheral initialization */

#if defined(__SILICON_REVISION__) && __SILICON_REVISION__==0x0
    /* kernel anomaly workaround */
    pBootConfig->pSource+=16;
#endif

    /* 1. Set the boot type to secure */
    pBootConfig->bootType=ADI_ROM_SECURE_BOOT;

    /* 2. Set they key type to custom ( this disables reading the key from OTP ) */
    pBootConfig->keyType = ADI_ROM_CUSTOM_SECURITY;

    /* 3. load the key to be used, this is the key that would normally be read from OTP
     *
     *  Choose the appropriate image type
     *
     * */
    //configureBlw(pBootConfig);
    configureBlx(pBootConfig);
    //configureBlp(pBootConfig);

  } else {
    /* executed after the initial boot kernel configuration is completed */
    setupMemoryMappedSPI(pBootConfig);
  }
  return 0;

}

void adi_mmu_Init()
{
	asm("nop;");
}


#if   defined(__ADSPBF70x__) && __ADSPBF70x__== 1
#define SPIADDR 0x0
#elif defined(__ADSPSC5xx__) && __ADSPSC5xx__== 1
#define SPIADDR 0x60000000
#endif
/**
 *
 * @brief The application loaded into L1 using the emulator
 *
 * */
int main(void)
{

#if defined( __ADSPBF70x__) && __ADSPBF70x__ == 1
  *pREG_L1IM_ICTL = 1;
#endif



  /* call the ROM API with peripheral mode and noauto cleared, and the secureboot hook function set */

  uint32_t spimBootCommand = ((ENUM_ROM_BCMD_DEVICE_SPI) |
                              (0x2 << BITP_ROM_BCMD_SPIM_SPEED)   |
                              (0x2 << BITP_ROM_BCMD_SPIM_ADDR)    |
                              (0x1 << BITP_ROM_BCMD_SPIM_BCODE)   |
                              (ENUM_ROM_BCMD_DEVENUM_2));

  adi_rom_Boot(SPIADDR,BITM_ROM_BFLAG_HOOK,0,&ConfigureForSecureBoot,spimBootCommand);


  /* At this point, call to the rom code would have secure-booted the boot stream in flash and the processor
   * would not reach here.
   */
  while(1);

}



